<?php

namespace QUI\ERP\CRM;

use Exception;
use QUI;

use function date;
use function is_int;

/**
 * Class Handler
 *
 * @package QUI\ERP\CRM
 */
class Task
{
    /**
     * @var integer
     */
    protected int $id;

    /**
     * @var array
     */
    protected array $data;

    /**
     * @var ?QUI\ERP\Comments
     */
    protected ?QUI\ERP\Comments $Comments = null;

    /**
     * Task constructor.
     *
     * @param int $taskId
     *
     * @throws TaskException
     */
    public function __construct(int $taskId)
    {
        $this->id = $taskId;

        try {
            $this->data = TaskHandler::getTaskData($this->id);
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::addAlert($Exception->getMessage());

            throw new TaskException(['quiqqer/erp', 'exception.task.error']);
        }

        // comments
        $this->Comments = new QUI\ERP\Comments();

        if (isset($this->data['comments'])) {
            $this->Comments = QUI\ERP\Comments::unserialize($this->data['comments']);
        }
    }

    /**
     * @return array
     */
    public function toArray(): array
    {
        return [
            'id' => $this->getId(),
            'title' => $this->getTitle(),
            'text' => $this->getDescription(),
            'type' => $this->getType(),
            'to' => $this->getTo(),
            'from' => $this->getFrom(),
            'userId' => $this->getUserId(),
            'editorId' => $this->getEditorId(),
            'status' => $this->getStatus(),
            'comments' => $this->getComments()->toArray()
        ];
    }

    /**
     * saves the current task
     */
    public function save()
    {
        try {
            QUI::getDataBase()->update(
                TaskHandler::table(),
                [
                    'title' => $this->getTitle(),
                    'text' => $this->getText(),
                    'taskType' => $this->getType(),
                    'to' => $this->getTo(),
                    'from' => $this->getFrom(),
                    'userId' => $this->getUserId(),
                    'editorId' => $this->getEditorId(),
                    'status' => $this->getStatus(),
                    'comments' => $this->getComments()->toJSON()
                ],
                ['id' => $this->getId()]
            );
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }

    //region getter

    /**
     * @return int
     */
    public function getId(): int
    {
        return $this->id;
    }

    /**
     * Return the title of the task
     *
     * @return string
     */
    public function getTitle(): string
    {
        return $this->data['title'];
    }

    /**
     * Return the text / description of the task
     *
     * @return string
     */
    public function getDescription(): string
    {
        return $this->data['text'];
    }

    /**
     * Alias for getDescription()
     *
     * @return string
     */
    public function getText(): string
    {
        return $this->getDescription();
    }

    /**
     * @return mixed
     */
    public function getType(): mixed
    {
        return $this->data['taskType'];
    }

    /**
     * @return mixed
     */
    public function getTo(): mixed
    {
        return $this->data['to'];
    }

    /**
     * @return mixed
     */
    public function getFrom(): mixed
    {
        return $this->data['from'];
    }

    /**
     * @return mixed
     */
    public function getStatus(): mixed
    {
        return $this->data['status'];
    }

    //endregion

    //region setter

    /**
     * @param string $title
     */
    public function setTitle(string $title): void
    {
        $this->data['title'] = $title;
    }

    /**
     * @param string $description
     */
    public function setDescription(string $description): void
    {
        $this->data['text'] = $description;
    }

    /**
     * @param string $description
     */
    public function setText(string $description): void
    {
        $this->setDescription($description);
    }

    /**
     * @param string $type
     */
    public function setType(string $type): void
    {
        $this->data['taskType'] = $type;
    }

    /**
     * @param integer|string $to
     */
    public function setTo(int|string $to): void
    {
        // unix timestamp
        if (is_int($to)) {
            $to = date('Y-m-d H:i:s', $to);
        }

        if (!QUI\Utils\Security\Orthos::checkMySqlDatetimeSyntax($to)) {
            return;
        }

        $this->data['to'] = $to;
    }

    /**
     * @param integer|string $from
     */
    public function setFrom(int|string $from): void
    {
        // unix timestamp
        if (is_int($from)) {
            $from = date('Y-m-d H:i:s', $from);
        }

        if (!QUI\Utils\Security\Orthos::checkMySqlDatetimeSyntax($from)) {
            return;
        }

        $this->data['from'] = $from;
    }

    /**
     * @param string $status
     */
    public function setStatus(string $status): void
    {
        $this->data['status'] = $status;
    }

    //endregion

    //region user

    /**
     * @return int|null
     */
    public function getUserId(): ?int
    {
        if (empty($this->data['userId'])) {
            return null;
        }

        return (int)$this->data['userId'];
    }

    /**
     * @return ?QUI\Interfaces\Users\User
     */
    public function getUser(): ?QUI\Interfaces\Users\User
    {
        if ($this->getUserId() === null) {
            return null;
        }

        try {
            return QUI::getUsers()->get($this->getUserId());
        } catch (Exception) {
            return null;
        }
    }

    /**
     * @param int|string $userId
     */
    public function setUserId(int|string $userId): void
    {
        try {
            $User = QUI::getUsers()->get($userId);
        } catch (Exception) {
            return;
        }

        $this->data['userId'] = $User->getUUID();
    }

    //endregion

    //region editor

    /**
     * @return int|string|null
     */
    public function getEditorId(): null|int|string
    {
        if (empty($this->data['editorId'])) {
            return null;
        }

        return (int)$this->data['editorId'];
    }

    /**
     * @return ?QUI\Interfaces\Users\User
     */
    public function getEditor(): ?QUI\Interfaces\Users\User
    {
        if ($this->getEditorId() === null) {
            return null;
        }

        try {
            return QUI::getUsers()->get($this->getEditorId());
        } catch (Exception) {
            return null;
        }
    }

    /**
     * @param string|int $editorId
     */
    public function setEditorId(string|int $editorId): void
    {
        try {
            $User = QUI::getUsers()->get($editorId);
        } catch (Exception) {
            return;
        }

        $this->data['editorId'] = $User->getUUID();
    }

    //endregion

    //region comments

    /**
     * @return ?QUI\ERP\Comments
     */
    public function getComments(): ?QUI\ERP\Comments
    {
        return $this->Comments;
    }

    /**
     * Add a comment to the task
     *
     * @param $text
     */
    public function addComment($text): void
    {
        $this->Comments->addComment(
            $text,
            false,
            'quiqqer/crm',
            'fa fa-edit'
        );
    }

    //endregion
}
