<?php

namespace QUI\ERP\CRM;

use Exception;
use PDO;
use QUI;

use function date;
use function explode;
use function implode;
use function is_numeric;
use function strtoupper;
use function trim;

/**
 * Class TaskSearch
 *
 * @package QUI\ERP\CRM
 */
class TaskSearch
{
    /**
     * @var array
     */
    protected array $filter = [];

    /**
     * @var array
     */
    protected array $limit = [];

    /**
     * @var bool
     */
    protected bool $issuable = false;

    /**
     * @var bool
     */
    protected string|bool $order = '`from` DESC';

    /**
     * TaskSearch constructor.
     */
    public function __construct()
    {
    }

    /**
     * @param string $search
     * @return array
     *
     * @throws TaskException
     */
    public function search(string $search): array
    {
        // build the query
        $query = 'SELECT * FROM ' . TaskHandler::table();
        $where = [];
        $binds = [];

        if (!empty($this->filter)) {
            $i = 0;

            foreach ($this->filter as $filter => $val) {
                $value = $val[0];
                $filterType = $val[1];
                $paramType = $val[2];

                $where[] = '(`' . $filter . '` ' . $filterType . ' :filter' . $i . ' OR `' . $filter . '` IS NULL)';

                $binds['filter' . $i] = [
                    'value' => $value,
                    'type' => $paramType
                ];

                $i++;
            }
        }

        if (!empty($search)) {
            $where[] = '(
                title LIKE :search OR 
                text LIKE :search
            )';

            $binds['search'] = [
                'value' => '%' . $search . '%',
                'type' => PDO::PARAM_STR
            ];
        }

        if ($this->issuable) {
            $where[] = '(`from` >= :now OR  `to` <= :now)';

            $binds['now'] = [
                'value' => date('Y-m-d H:i:s'),
                'type' => PDO::PARAM_STR
            ];
        }

        if (!empty($where)) {
            $query .= ' WHERE ' . implode(' AND ', $where);
        }

        if ($this->order) {
            $query .= ' ORDER BY ' . $this->order;
        }

        if ($this->limit) {
            $query .= ' LIMIT ' . $this->limit[0] . ',' . $this->limit[1];
        }

        // exec query
        $PDO = QUI::getDataBase()->getPDO();
        $Statement = $PDO->prepare($query);

        foreach ($binds as $var => $bind) {
            $Statement->bindValue($var, $bind['value'], $bind['type']);
        }

        try {
            $Statement->execute();

            return $Statement->fetchAll(PDO::FETCH_ASSOC);
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            QUI\System\Log::writeRecursive($query);
            QUI\System\Log::writeRecursive($binds);
            throw new TaskException('Something went wrong');
        }
    }

    /**
     * @return int
     * @throws TaskException
     */
    public function count(): int
    {
        // build the query
        $query = 'SELECT COUNT(id) as c FROM ' . TaskHandler::table();
        $where = [];
        $binds = [];

        if (!empty($this->filter)) {
            $i = 0;

            foreach ($this->filter as $filter => $val) {
                $value = $val[0];
                $filterType = $val[1];
                $paramType = $val[2];

                $where[] = '(`' . $filter . '` ' . $filterType . ' :filter' . $i . ' OR `' . $filter . '` IS NULL)';

                $binds['filter' . $i] = [
                    'value' => $value,
                    'type' => $paramType
                ];

                $i++;
            }
        }

        if ($this->issuable) {
            $where[] = '(`from` >= :now OR  `to` <= :now)';

            $binds['now'] = [
                'value' => date('Y-m-d H:i:s'),
                'type' => PDO::PARAM_STR
            ];
        }

        if (!empty($this->search)) {
            $where[] = '(
                title LIKE :search OR 
                text LIKE :search
            )';

            $binds['search'] = [
                'value' => '%' . $this->search . '%',
                'type' => PDO::PARAM_STR
            ];
        }

        if (!empty($where)) {
            $query .= ' WHERE ' . implode(' AND ', $where);
        }

        // exec query
        $PDO = QUI::getDataBase()->getPDO();
        $Statement = $PDO->prepare($query);

        foreach ($binds as $var => $bind) {
            $Statement->bindValue($var, $bind['value'], $bind['type']);
        }

        try {
            $Statement->execute();

            $result = $Statement->fetchAll(PDO::FETCH_ASSOC);
        } catch (Exception $Exception) {
            QUI\System\Log::writeException($Exception);
            QUI\System\Log::writeRecursive($query);
            QUI\System\Log::writeRecursive($binds);
            throw new TaskException('Something went wrong');
        }

        return (int)$result[0]['c'];
    }

    //region setter

    /**
     * @param $start
     * @param $size
     */
    public function setLimit($start, $size): void
    {
        $this->limit = [(int)$start, (int)$size];
    }

    /**
     * @param string $order
     */
    public function setOrder(string $order): void
    {
        $order = explode(' ', $order);
        $field = trim($order[0]);
        $field = trim($field, '`');

        $sort = 'DESC';

        if (isset($order[1])) {
            $order[1] = strtoupper($order[1]);

            switch ($order[1]) {
                case 'ASC':
                case 'DESC':
                    $sort = $order[1];
            }
        }

        switch ($field) {
            case 'from':
            case 'to':
            case 'id':
            case 'userId':
            case 'editorId':
            case 'status':
                break;

            default:
                $field = 'from';
        }

        $this->order = '`' . $field . '` ' . $sort;
    }

    //endregion

    //region filter

    /**
     * Set a filter
     *
     * @param string $name
     * @param int|string $value
     * @param string $type - optional: =, >, <, <=, >=
     * @param int $paramType - optional: PDO PARAM const, eq: PDO::PARAM_STR
     */
    public function setFilter(
        string $name,
        int|string $value,
        string $type = '=',
        int $paramType = PDO::PARAM_STR
    ): void {
        switch ($paramType) {
            case PDO::PARAM_NULL:
            case PDO::PARAM_STR:
            case PDO::PARAM_INT:
            case PDO::PARAM_BOOL:
                break;

            default:
                $paramType = PDO::PARAM_STR;
        }

        if ($name === 'from' && is_numeric($value)) {
            $value = date('Y-m-d H:i:s', (int)$value);
        }

        if ($name === 'to' && is_numeric($value)) {
            $value = date('Y-m-d H:i:s', (int)$value);
        }

        $this->filter[$name] = [$value, $type, $paramType];
    }

    /**
     * Clears all filter
     * Reset all filter
     */
    public function clearFilter(): void
    {
        $this->filter = [];
    }

    /**
     * set the issuable flag to true
     * So, only issues which has to be done will be shown
     */
    public function onlyIssuable(): void
    {
        $this->issuable = true;
    }

    /**
     * set the issuable flag to false
     * So, all issues will be shown, not only issues which has to be done
     */
    public function resetOnlyIssuable(): void
    {
        $this->issuable = false;
    }

    //endregion
}
